// class header
#include "tabview.h"

// includes
#include "TextContent.h"
#include "Style.h"
#include <QGraphicsScene>
#include <QGraphicsTextItem>
#include <QPropertyAnimation>
#include <QParallelAnimationGroup>
#include <QMouseEvent>
#include <QDebug>

//  constants
const int itemCount = 6;

TabView::TabView(QGraphicsScene *scene,QWidget *parent) :
   QGraphicsView(scene,parent),
    mCurrentItem(0)
{
    // set blackish background
    setStyleSheet(backgroundStyle);

    // create opacity animator
    mOpacityAnimator = new QPropertyAnimation();
    mOpacityAnimator->setDuration(animationTime);
    mOpacityAnimator->setPropertyName("opacity");
    mOpacityAnimator->setEasingCurve( QEasingCurve::Linear );
    mOpacityAnimator->setStartValue(0.3);
    mOpacityAnimator->setEndValue(1.0);

    // create animation groups
    mGroupAnimHeader = new QParallelAnimationGroup;;
    mGroupAnimContent = new QParallelAnimationGroup;

    createTopBar();
    createMetroTabBar();
    createContentItems();
}

TabView::~TabView()
{
    delete mGroupAnimContent;
    delete mGroupAnimHeader;
    qDeleteAll(mContentItems);
    qDeleteAll(mHeaderItems);
    mHeaderAnimations.clear();
    mContentAnimations.clear();
}

void TabView::createTopBar()
{
    QGraphicsTextItem *bar = new QGraphicsTextItem();
    bar->setAcceptHoverEvents(false);
    bar->setFont(bodyFont);
    bar->setPlainText("Qt Commercial Blog");
    bar->setDefaultTextColor(uiTextColor);
    bar->setPos(componentMargin, componentMargin);
    scene()->addItem(bar);
}

void TabView::createMetroTabBar()
{
    QGraphicsTextItem *tmp;
    QPropertyAnimation *anim;
    int xPos = componentMargin;
    QString text;

    // create 6 items
    for(int i = 0; i < itemCount; ++i) {
        tmp = new QGraphicsTextItem();
        tmp->setAcceptHoverEvents(false);
        text = "loremIpsum";
        text = text.append(QString("%1").arg(i + 1));
        tmp->setPlainText(text);
        tmp->setFont(headerFont);
        tmp->adjustSize();
        tmp->setDefaultTextColor(uiTextColor);

        // below header text
        tmp->setPos(xPos, (componentMargin * 2 + bodyTextSize));

        // calculate position for the next item. ComponentMargin + item width + ComponentMargin
        xPos = xPos + tmp->textWidth() + componentMargin;

        anim = new QPropertyAnimation(tmp,"pos");
        anim->setDuration(animationTime);
        anim->setPropertyName("pos");
        anim->setEasingCurve(QEasingCurve::OutCirc);
        mGroupAnimHeader->addAnimation(anim);
        mHeaderAnimations.append(anim);

        // de-highlight other than first item
        if(i != 0)
            tmp->setOpacity(0.3);

        mHeaderItems.append(tmp);
        scene()->addItem(tmp);
    }
}

void TabView::createContentItems()
{
    QGraphicsTextItem *tmp;
    QPropertyAnimation *anim;
    QString text;
    int xPos = componentMargin;
    // scene width - margins
    int textWidth = scene()->width() - componentMargin - componentMargin;

    // create 6 items
    for(int i = 0; i < itemCount; ++i) {
        tmp = new QGraphicsTextItem();
        tmp->setFont(bodyFont);
        tmp->setAcceptHoverEvents(false);
        text = TextContent::getLorem(i);
        tmp->setPlainText(text);
        tmp->setTextWidth(textWidth);
        tmp->setDefaultTextColor(uiTextColor);

        // below header and pivot
        tmp->setPos(xPos,(componentMargin * 4 + bodyTextSize + headerTextSize));

        // calculate the position for the next item. ComponentMargin + item width + ComponentMargin
        xPos = xPos + textWidth + componentMargin;

        anim = new QPropertyAnimation(tmp,"pos");
        anim->setDuration(animationTime);
        anim->setPropertyName("pos");
        anim->setEasingCurve(QEasingCurve::OutCirc);

        mGroupAnimContent->addAnimation(anim);
        mContentAnimations.append(anim);
        mContentItems.append(tmp);
        scene()->addItem(tmp);
    }
}

void TabView::mousePressEvent(QMouseEvent *event)
{
     mMouseXPosition = event->pos().x();
}

void TabView::mouseMoveEvent(QMouseEvent *event)
{
    // negative is to left and positive is to right
    int xDif = event->pos().x() - mMouseXPosition;

    // don't start animations twice
    if(mGroupAnimContent->state() != QAbstractAnimation::Running
       && mGroupAnimHeader->state() != QAbstractAnimation::Running) {
        // sweep <--
        if(xDif < 0) {
            // don't get over the edge
            if(mCurrentItem + 1 < itemCount) {
                mCurrentItem++;
                startContentAnimation(ESweepLeft);
                startHeaderAnimation(ESweepLeft);
            }
        // sweep -->
        } else if(xDif > 0) {
            // don't get over the edge
            if(mCurrentItem - 1 >= 0) {
                mCurrentItem--;
                startContentAnimation(ESweepRight);
                startHeaderAnimation(ESweepRight);
            }
        }
    }
    mMouseXPosition = event->pos().x();
}

void TabView::startContentAnimation(int direction)
{
    QPropertyAnimation *anim;
    QGraphicsTextItem  *text;

    // init animation items
    for(int i = 0; i < itemCount; ++i) {
        text = mContentItems.at(i);
        anim = mContentAnimations.at(i);
        QPointF start = text->pos();
        QPointF end = start;

        if(direction == ESweepLeft)
            end.setX( end.x() - text->textWidth() - componentMargin);
        else
            end.setX( end.x() + text->textWidth() + componentMargin);

        anim->setStartValue(start);
        anim->setEndValue(end);
    }
    mGroupAnimContent->start();
}

void TabView::startHeaderAnimation(int direction)
{
    QPropertyAnimation *anim;
    QGraphicsTextItem *text;

    // init animation items
    for(int i = 0; i < itemCount; ++i) {
        text = mHeaderItems.at(i);
        anim = mHeaderAnimations.at(i);
        QPointF start = text->pos();
        QPointF end = start;

        if(direction == ESweepLeft)
            end.setX( end.x() - text->textWidth() - componentMargin);
        else
            end.setX( end.x() + text->textWidth() + componentMargin);

        anim->setStartValue(start);
        anim->setEndValue(end);
        // reset opacity
        text->setOpacity(0.3);
    }
    mOpacityAnimator->setTargetObject(mHeaderItems.at(mCurrentItem));
    mGroupAnimHeader->start();
    mOpacityAnimator->start();
}
